'''
Here we'll define exceptions to raise in the qtstyles package
'''

class QtStylesError(Exception):
    """ Base-class for all exceptions raised by this module. """


class SheetPathTypeError(QtStylesError):
    ''' The style sheet path must be a string '''
    def __init__(self):
        super(SheetPathTypeError, self).__init__("The style sheet path must be a string!")


class SheetPathValueError(QtStylesError):
    ''' The style sheet path end in '.qss' '''
    def __init__(self):
        super(SheetPathValueError, self).__init__("The style sheet path must end in '.qss'!")


class SheetPathFileDoesntExist(QtStylesError):
    ''' The style sheet path must point to a file that exists '''
    def __init__(self):
        super(SheetPathFileDoesntExist, self).__init__("No file exists at the path specified!")


class StyleDoesntExistError(QtStylesError):
    ''' The provided style sheet path must exist '''
    def __init__(self):
        super(StyleDoesntExistError, self).__init__("The requested style does not exist!")



'''
Defines only the StylePickerWidget class which inherits from QComboBox
'''

from qtpy import QtWidgets
from qtstyles.picker import StylePicker


class StylePickerWidget(QtWidgets.QComboBox):
    ''' This QComboBox will allow the user to change the application
        style sheet on demand '''
    def __init__(self, parent=None):
        ''' constructor only takes the parent widget as an argument '''
        super(StylePickerWidget, self).__init__(parent)
        self.addItems(StylePicker().available_styles)
        self.setCurrentIndex(self.findText("default"))
        self.currentIndexChanged.connect(self.change_app_style)

    def change_app_style(self):
        ''' this method fires when a new style is selected '''
        style = self.currentText()
        app = QtWidgets.QApplication.instance()
        if app is not None:
            app.setStyleSheet(StylePicker(style).get_sheet())


"""
def main():
    ''' a quick demo of our StylePickerWidget class '''
    app = QtWidgets.QApplication.instance()
    if not app:
        app = QtWidgets.QApplication([])

    win = QtWidgets.QMainWindow()
    win.setWindowTitle("Test StylePickerWidget")
    picker_widget = StylePickerWidget()
    win.setCentralWidget(picker_widget)
    win.show()
    app.exec_()

if __name__ == "__main__":
    main()
"""



'''
Includes a run_demo() function that acts as a basic demo for our
StylePicker and StylePickerWidget classes
'''

import random
from qtpy import QtWidgets, QtCore
from qtstyles.picker import StylePicker
from qtstyles.widget import StylePickerWidget


def run_demo(close_after=None, auto_test=False):
    '''
    Args:
    close_after: either None or integer. Number of seconds
    after which the demo will close
    auto_test: boolean. If true then randomly select a style at intervals.

    This function provides a demonstration in changing the application style sheet.
    1) The first option is to set the style sheet once with
       app.setStyleSheet(StylePicker("default").get_sheet())
    2) The second option is to include a style picker widget (QComboBox) that'll
       change the application style sheet when a new style is selected
       grid.addWidget(StylePickerWidget(), 2, 0)
    '''

    app = QtWidgets.QApplication.instance()
    if not app:
        app = QtWidgets.QApplication([])

    win = QtWidgets.QMainWindow()
    win.setWindowTitle("Style Sheets")

    frame = QtWidgets.QFrame()
    win.setCentralWidget(frame)

    grid = QtWidgets.QGridLayout(frame)
    grid.setHorizontalSpacing(5)
    grid.setVerticalSpacing(5)
    grid.addWidget(QtWidgets.QLabel("Username"), 0, 0)
    grid.addWidget(QtWidgets.QLabel("Password"), 1, 0)
    user_input = QtWidgets.QLineEdit()
    pass_input = QtWidgets.QLineEdit()
    grid.addWidget(user_input, 0, 1)
    grid.addWidget(pass_input, 1, 1)

    picker_widget = StylePickerWidget()
    grid.addWidget(picker_widget, 2, 0)
    grid.addWidget(QtWidgets.QPushButton("Submit"), 2, 1)
    win.show()

    def choose_random_style():
        ''' select a random style from the picker_widget '''
        style_list = StylePicker().available_styles
        chosen_style = random.choice(style_list)
        picker_widget.setCurrentIndex(picker_widget.findText(chosen_style))

    def close_demo():
        ''' close the demo once 'close_after' seconds have elapsed '''
        win.close()

    if auto_test:
        timer = QtCore.QTimer()
        timer.timeout.connect(choose_random_style)
        timer.start(1000)

    if isinstance(close_after, int):
        close_timer = QtCore.QTimer()
        close_timer.singleShot(close_after * 1000, close_demo) # seconds * 1000 = milliseconds

    app.setStyleSheet(StylePicker("default").get_sheet())
    app.exec_()

"""
def main():
    ''' testing our run_demo function '''
    run_demo(close_after=10, auto_test=True)

if __name__ == "__main__":
    main()
"""


'''
Defines the following ...

Sheet: a class representing a style sheet object with attributes such as path and contents

get_style_sheets: a function that returns a dictionary with style sheet names as keys and
sheet objects as values
'''

import os
from qtstyles import errors


class Sheet(object):
    '''
    Keeps key information related to style sheets, particularly the path
    and contents as a string.
    >>> import os
    >>> dirpath = os.path.dirname(os.path.abspath(__file__))
    >>> path = os.path.join(dirpath, "style_sheets", "default.qss")
    >>> sheet = Sheet(path)
    >>> isinstance(sheet.contents, str)
    True
    '''
    def __init__(self, path):
        ''' This constructor only takes one argument being the sheet path.
            path = the path to a qss file... must be a string and end with '.qss'
        '''
        if not isinstance(path, str):
            raise errors.SheetPathTypeError
        if not path.endswith(".qss"):
            raise errors.SheetPathValueError
        if not os.path.isfile(path):
            raise errors.SheetPathFileDoesntExist
        self._path = path
        self._contents = None # to be loaded on request

    @property
    def path(self):
        ''' collect the path as a sheet attribute '''
        return self._path

    @property
    def contents(self):
        ''' the style sheet contents will load only once when needed '''
        if self._contents is None:
            self._load_contents()
        return self._contents

    def _load_contents(self):
        ''' loads the style sheet contents (if not already loaded) '''
        with open(self.path, "r") as qss_file:
            self._contents = qss_file.read()


def get_style_sheets():
    '''
    Returns a dictionary with the style sheet names as keys and
    associated Sheet objects as values
    There must be a sheet called 'default' which is empty.
    >>> sheets = get_style_sheets()
    >>> isinstance(sheets, dict) # returns a dictionary
    True
    >>> sheet_object = sheets["default"] # there should be a 'default.qss' sheet available
    >>> sheet_object.path.endswith(".qss") # these should all be .qss files
    True
    '''
    dirpath = os.path.dirname(os.path.abspath(__file__))

    sheets = {}
    for name in os.listdir(os.path.join(dirpath, "style_sheets")):
        if "__" in name:
            # exclude any files with a double underscore (e.g. __init__, __pycache__)
            continue
        path = os.path.join(dirpath, "style_sheets", name)
        sheets[name.replace(".qss", "")] = Sheet(path)
    return sheets



'''
Defines StylePicker: a class that allows us to pick style sheets and collect their
style sheet contents with get_sheet()
'''

from qtstyles import errors
from qtstyles.sheet import get_style_sheets


class StylePicker(object):
    '''
    The StylePicker class has the following properties:
    style: the current selected style
    available_styles: a list of strings representing all of the styles available

    And the following methods:
    __init__(style='default'): the constructor takes one argument being the selected style
    get_sheet(): returns a string being the style sheet contents for our selected style

    'sheets' is a class attribute that'll act as a shared resource for all StylePicker
    objects. There is no point fetching the sheets data multiple times as it is static.
    '''
    sheets = get_style_sheets()

    def __init__(self, style ="default"):
        self.style = style

    def get_sheet(self):
        ''' returns a string containing the style sheet for our chosen style '''
        return self.sheets[self.style].contents

    @property
    def style(self):
        ''' gets the current selected style '''
        return self._style

    @style.setter
    def style(self, style):
        ''' sets the current style which must be in the available_styles list '''
        if style not in self.available_styles:
            raise errors.StyleDoesntExistError
        self._style = style

    @property
    def available_styles(self):
        ''' returns a list of all available styles we can choose from
            Note that dict.keys() will return a list in py2 but a dict_keys object in py3 '''
        return list(self.sheets.keys())



'''
Qt based functionality such as the StylePickerWidget and run_demo
will not be imported if qtpy is not installed.
'''

import os
import warnings
import random

try:
    from qtpy import QtWidgets, QtCore
except ImportError:
    warnings.warn("qtstyles Qt functionality requires qtpy to be installed.", ImportWarning)
else:
    from .demo import run_demo
    from .widget import StylePickerWidget

from .sheet import Sheet, get_style_sheets
from .picker import StylePicker



from setuptools import setup, find_packages

setup(name="qtstyles",
      version="0.0.1",
      install_requires=[
          "QtPy>=1.4.1"
        ],
      description="A collection of Qt style sheets and helpful classes for applying them.",
      long_description=open("README.md").read(),

      # https://setuptools.readthedocs.io/en/latest/setuptools.html#including-data-files
	  package_data={"qtstyles": ["style_sheets/*.qss"]}, # include style sheets
      author="Simon Garisch",
      author_email="gatman946@gmail.com",
      url="https://github.com/simongarisch/qtstyles",
      packages=find_packages()
     )
